% Input f contains the fixed coefficients, and has dimension NFX1.
% Input c contains the random coefficients for each person, and has dimensions NV x NP.
% Either input can be an empty matrix. 
% Output p contains the logit probabilities, which is a row vector of dimension 1xNP.
% Output g contains the gradients of log(p), which is a matrix (NF+NV+NV) x NP
% Code assumes that all GLOBALS already exist.


function [p, g]=llgrad2(f,b,w) 

global NV NF NP NDRAWS NALTMAX NCSMAX NMEM NTAKES
global X S XF DR corrc
global MDR

small=1e-321;

NVP=length(w); %Number of variance parameters

p=zeros(1,NP);  %Row vector to hold probability of options chosen
g=zeros(NF+NV+NVP,NP);   %Matrix to hold hh level gradients

for r=1:NTAKES   %NTAKES is the number of draws that are are read into memory at one time                              %NTAKE=NDRAWS./NMEM. These must be divisable as an integer. 

if NTAKES>1
   DR=MDR.Data(1).drs((r-1).*NMEM+1:r.*NMEM,:,:); %Dimensions: NMEM x NP x NV
   DR=permute(DR,[3,2,1]); %To make NV x NP x NMEM
end

%Recall:
% X  is a NALTMAX-1xNCSMAXxNVxNP Matrix of 'random' variables 
% XF is a NALTMAX-1xNCSMAXxNFxNP Matrix of 'fixed' variables

c=trans(b,w,DR,NP);   %c is NVxNPxNMEM matrix of b+w*dr (possibly transformed)
v=zeros(NMEM,NALTMAX-1,NCSMAX,NP);  %NMEMxNALTMAX-1xNCSMAXxNP matrix to hold index 
if NF > 0
   ff=reshape(f,1,1,NF,1);  %1x1xNFx1 matrix of fixed coeffs
   ff=repmat(ff,[NALTMAX-1,NCSMAX,1,NP]); %NAM-1xNCSMxNFxNP matrix of fixed coeffs
   vf=reshape(sum(XF.*ff,3),NALTMAX-1,NCSMAX,NP);  %NAM-1xNCSMxNP matrix of f*FX
else
   vf=zeros(NALTMAX-1,NCSMAX,NP);
end
vf=repmat(vf,[1,1,1,NMEM]); %NAM-1xNCSMxNPxNMEM matrix of f*FX

if NV >0
   
   cc=reshape(c,1,1,NV,NP,NMEM); %1x1xNVxNPxNMEM matrix of b+w*dr
   cc=repmat(cc,[NALTMAX-1,NCSMAX,1,1,1]); %NAM-1xNCSMxNVxNPxNMEM matrix of b+w*dr
   v=(repmat(X,[1,1,1,1,NMEM]).*cc); %NAM-1xNCSMxNVxNPxNMEM matrix of (b+w*dr).*RX
   v=reshape(sum(v,3),NALTMAX-1,NCSMAX,NP,NMEM); %NAM-1xNCSMxNPxNMEM matrix of (b+w*dr)*RX
   v=v+vf; %NAM-1xNCSMxNPxNMEM matrix of f*FX+(b+w*dr)*RX
else
   v=vf;
end

v=exp(v);  %NAM-1xNCSMxNPxNMEM matrix of exp(f*FX+(b+w*dr)*RX)
v(isinf(v))=exp(600);  %As precaution when exp(v) is too large for machine
v=v.*repmat(S,[1,1,1,NMEM]);  %Set exp(v) to zero for non-options/CS
pp=1./(1+sum(v,1)); %1xNCSMAXxNPxNMEM matrix of probs for options chosen

%Calculate gradient

  gg=v.*repmat(pp,[NALTMAX-1,1,1,1]);   %NAM-1xNCSMxNPxNMEM matrix of probs for all nonchosen alts
  gg=reshape(gg,NALTMAX-1,NCSMAX,1,NP,NMEM); %NAM-1xNCSMx1xNPxNMEM
  if NF>0
      %NAM-1xNCSMxNFxNPxNMEM matrix of FX*prob_{-j*}
      grf=-repmat(gg,[1,1,NF,1,1]).*repmat(XF,[1,1,1,1,NMEM]);
      %NFxNPxNMEM sum across options and trips of FX*prob_{-j*}
      grf=reshape(sum(sum(grf,1),2),NF,NP,NMEM);   %NFxNPxNMEM
  else
      grf=[];
  end
   
  if NV>0
      %NAM-1xNCSMxNVxNPxNMEM matrix of RX*prob_{-j*}
      gg=-repmat(gg,[1,1,NV,1,1]).*repmat(X,[1,1,1,1,NMEM]);
      %If random coeff is not normal, must calculate the derivative of the
      %transformation wrt the parameters - done by der
      %grb is (NVxNPxNMEM) dtran/db (eg exp(b+w*dr) if log normal)
      %grw is (NVxNPxNMEM) dtran/dw (eg dr*exp(b+w*dr) if log normal)
      [grb, grw]=der(b,w,DR);
      grb=reshape(grb,1,1,NV,NP,NMEM);% (1x1xNVxNPxNMEM) dtran/db
      if corrc==1
          grw=reshape(grw,1,1,0.5*NV*(NV+1),NP,NMEM);% (1x1xNVxNPxNMEM) dtran/dw
      %NAM-1xNCSMxNVxNPxNMEM matrix of RX*prob_{-j*}*dtran/db
      elseif corrc==0
          grw=reshape(grw,1,1,NV,NP,NMEM);% (1x1xNVxNPxNMEM) dtran/dw
      %NAM-1xNCSMxNVxNPxNMEM matrix of RX*prob_{-j*}*dtran/db
      end
      grb=gg.*repmat(grb,[NALTMAX-1,NCSMAX,1,1,1]);
      %NAM-1xNCSMxNVPxNPxNMEM matrix of RX*prob_{-j*}*dtran/dw
      if corrc==1
        grw2=zeros(NALTMAX-1,NCSMAX,NVP,NP,NMEM);
        x=0;
        for i=1:NV
            for j=1:i
                grw2(:,:,x+1,:,:)=gg(:,:,i,:,:).*repmat(grw(:,:,x+1,:,:),[NALTMAX-1,NCSMAX,1,1,1]);
                x=x+1;
            end
        end
      elseif corrc==0
        grw2=gg.*repmat(grw,[NALTMAX-1,NCSMAX,1,1,1]);
      end
      
      %NVxNPxNMEM sum over options and cs of RX*prob_{-j*}*dtran/db
      grb=reshape(sum(sum(grb,1),2),NV,NP,NMEM);   %NVxNPxNMEM 
      %NVPxNPxNMEM sum over options and cs of RX*prob_{-j*}*dtran/dw
      grw2=reshape(sum(sum(grw2,1),2),NVP,NP,NMEM);   %NVxNPxNMEM
  else
      grb=[];
      grw2=[];
  end
%Back to prob
pp=reshape(pp,NCSMAX,NP,NMEM);     %NCSMAXxNPxNMEM
pp=prod(pp,1);      %1xNPxNMEM matrix of sequence of probs
gr=[grf;grb;grw2];   %(NF+NV+NV)xNPxNMEM
%Gradient
   gr=gr.*repmat(pp,[NF+NV+NVP,1,1]); %Multiply by product of probs
   g=g+sum(gr,3);  %(NF+NV+size(w))xNP sum over draws 

%Back to prob
pp=sum(pp,3);       %1xNP sum of draws

p=p+pp; %Sums over NTAKES

end

p=p./NDRAWS;    %Simulated sequence of choice probabilities of options chosen
p(1,isnan(p))=1; %Change missing values to 1, as a precaution. 
%Gradient
   g=g./NDRAWS; %Average grad so far across simulations 
   g=g./repmat(p+small,NF+NV+NVP,1); %And then devide by simulated product of probs